/* Celestial Empire (c) 1993 Dougal Scott */
/* List of potential planet names */
/* $Id: planet.h,v 1.1 1993/09/17 07:24:20 dwagon Exp $ */
/* $Header: /nelstaff/edp/dwagon/rfs/RCS/planet.h,v 1.1 1993/09/17 07:24:20 dwagon Exp $ */
/* $Log: planet.h,v $
 * Revision 1.1  1993/09/17  07:24:20  dwagon
 * Initial revision
 * */

#define NUMNAMES	1063

static char planetlist[NUMNAMES][NAMESIZ]={
"Aasarus",
"Abel",
"Aberius",
"Abyssinia",
"Aceldama",
"Achates",
"Achernar",
"Acheron",
"Achilles",
"Acrab",
"Acrisis",
"Acropolis",
"Adam",
"Adara",
"Adhara",
"Adonis",
"Adrastea",
"Adreaticus",
"Aeetes",
"Aegea",
"Aegir",
"Aeneas",
"Aeolus",
"Aesir",
"Aesop",
"Aether",
"Agamemnon",
"Agnar",
"Agni",
"Aiakos",
"Ajax",
"Akbar",
"Akroteri",
"Albion",
"Albireo",
"Alcmene",
"Alcor",
"Aldebran",
"Aleramin",
"Alexander",
"Alfard",
"Alferatz",
"Alfheim",
"Alfwecca",
"Algol",
"Alhema",
"Alioth",
"Alkalurops",
"Almach",
"Alnath",
"Alnitak",
"Alpha",
"Altair",
"Aludra",
"Alya",
"Amalthea",
"Amaterasu",
"Americos",
"Amphion",
"Anacreon",
"Ananke",
"Anchises",
"Andor",
"Andromeda",
"Angakok",
"Angurboda",
"Ansu",
"Antaeus",
"Antares",
"Anthony",
"Antigone",
"Antlia",
"Anubis",
"Apache",
"Apaosha",
"Aphrodite",
"Aphrodite",
"Apocalypse",
"Apollo",
"Apsyrtus",
"Aquarius",
"Aquila",
"Ara",
"Aragorn",
"Arapaho",
"Archernar",
"Arcturus",
"Ares",
"Argos",
"Argus",
"Ariadne",
"Ariel",
"Arioch",
"Arion",
"Armageddon",
"Artemis",
"Ascella",
"Asgard",
"Ashur",
"Astarte",
"Athens",
"Atlantis",
"Atlas",
"Atropos",
"Attica",
"Audhumla",
"Auriga",
"Avalon",
"Avior",
"Ayachi",
"Azagthoth",
"Azazel",
"AziDahak",
"Azores",
"Baal",
"Babylon",
"Bacchus",
"Baezelbub",
"Bagdemagus",
"Bakongo",
"Balder",
"Bali",
"Ban",
"Baugi",
"Bear",
"Bedivere",
"Beholder",
"Belial",
"Bellatrix",
"Bellerophon",
"Bellona",
"Benetnasch",
"Benkei",
"Bergelmir",
"Bernoulli",
"Berserks",
"Beta",
"Betelgeuse",
"Bianca",
"Bifrost",
"Boeotia",
"Bohort",
"Bohr",
"Bohr",
"Bones",
"Bootes",
"Bora",
"Boreas",
"Bors",
"Boz",
"Bragi",
"Brahe",
"Brave",
"Brin",
"Brisingamen",
"Britannia",
"Brust",
"Bujold",
"Buri",
"Bushido",
"Byron",
"Cadiz",
"Caeser",
"Cain",
"Callisto",
"Calrissian",
"Calypso",
"Camelot",
"Camille",
"Camlan",
"Canan",
"Cancer",
"Caneb",
"Canis",
"Canopus",
"Capella",
"Capricorn",
"Carbonek",
"Card",
"Carina",
"Carme",
"Carnicula",
"Caroli",
"Carthage",
"Cassiopeia",
"Castor",
"Cat",
"Celaeno",
"Celephais",
"Celeus",
"Celt",
"Centauri",
"Cepheus",
"Cerberus",
"Cetelus",
"Ceto",
"Cetus",
"Ch'eng-huang",
"Chansky",
"Chaos",
"Charon",
"Charybdis",
"Cheiron",
"Cherubim",
"Chi",
"Chi'ih-yu",
"Chimera",
"Chiron",
"Chow",
"Chuai",
"Cirrus",
"Citurnae",
"Claire",
"Cleito",
"Clotho",
"Clytemnestra",
"Cocytus",
"Colchis",
"Columba",
"Coniraya",
"Copernicus",
"Cordelia",
"Corinth",
"Coriolis",
"Corona",
"Corvus",
"Cranus",
"Crassus",
"Crater",
"Cressida",
"Cronus",
"Crusher",
"Crux",
"Cthulhu",
"Cursa",
"Curtis",
"Cyclops",
"Cygnus",
"Cylon",
"Cyzicus",
"Daedalus",
"Dagon",
"Dalos",
"Damon",
"Danae",
"Danaus",
"Dannaura",
"Danube",
"Daphne",
"Dardanus",
"Dark",
"Darkalheim",
"Darwin",
"Data",
"Dawn",
"Death",
"Deganus",
"Deimus",
"Delphi",
"Delphinus",
"Delta",
"Demeter",
"Demise",
"Deneb",
"Depth",
"Descartes",
"Desdemona",
"Desire",
"Desolation",
"Despoina",
"Deucalion",
"Devadatta",
"Dharma",
"Diana",
"Dinka",
"Diogenes",
"Dionysus",
"Diphda",
"Dis",
"Djanggawul",
"Donaldson",
"Dorado",
"Dorcas",
"Dorian",
"Dorus",
"Doula",
"Dove",
"Draco",
"Dragon",
"Drak",
"Draupnir",
"Dream",
"Druj",
"Dubhe",
"Dwarka",
"Dyaus",
"Ea",
"Earth",
"Echo",
"Ectoprimax",
"Ector",
"Eddas",
"Edjo",
"Eegos",
"Ehecatl",
"Einstein",
"Ekpe",
"Elara",
"Eldorado",
"Electra",
"Eleusis",
"Ellindil",
"Elrond",
"Elysia",
"Enceladus",
"Endoprimax",
"Eniph",
"Enkindu",
"Enlil",
"Ephialtes",
"Ephraites",
"Epigoni",
"Epimetheus",
"Epsilon",
"Erebus",
"Erech",
"Ergos",
"Eridanus",
"Erishkegal",
"Eros",
"Erusa",
"Eshu",
"Estanatleh",
"Eta",
"Eteocles",
"Eurasia",
"Euripedes",
"Europa",
"Eurydice",
"Eurytion",
"Excalibur",
"Exodus",
"Falcha",
"Fantor",
"Februs",
"FeiLien",
"Fenrir",
"Fermi",
"Fisher",
"Fomalhaut",
"Fomori",
"FonLegba",
"Ford",
"Fornax",
"Forseti",
"Fortuna",
"FourthWorld",
"Fredux",
"Freedom",
"Freuchen",
"Freyr",
"Frigg",
"Frisia",
"Fu-hsi",
"Fujin",
"Future",
"Gabora",
"Gades",
"Gadir",
"Gaea",
"Galactica",
"Galahad",
"Galanopoulus",
"Galileo",
"Gamma",
"Gamorae",
"Gandalf",
"Ganymede",
"Gaokerena",
"Garm",
"Gathas",
"Gauss",
"Gawain",
"Geinah",
"Geirrod",
"Gemini",
"Gerd",
"Gerda",
"Germania",
"Geryon",
"Ghal",
"Gil-galad",
"Gilgamesh",
"Gimli",
"Ginnungagap",
"Gjallarhorn",
"Glispa",
"Gnur",
"Gomeisa",
"Gondor",
"Gorgias",
"Gorgon",
"Gorlois",
"Gosadaya",
"Graeae",
"Granis",
"Grunbunde",
"Grus",
"Guiness",
"Guinevere",
"Gungnir",
"Hadad",
"Hadar",
"Halley",
"Halloway",
"Halvec",
"Hamal",
"Hamill",
"Haoma",
"Haroeris",
"Harpie",
"Hathor",
"Hawking",
"Heaven",
"Hebe",
"Hecate",
"Hector",
"Hegel",
"Hei-tiki",
"Heidegger",
"Heidrun",
"Heimdall",
"Heisenberg",
"Heisenburg",
"Hel",
"Helene",
"Hell",
"Helle",
"Hephiastes",
"Hera",
"Heracles",
"Herbert",
"Hercules",
"Hermod",
"Hesoid",
"Hestia",
"Himalia",
"Hodge",
"Hodur",
"Hoeinir",
"Hofstader",
"Holly",
"Homer",
"Hope",
"Hopper",
"Horae",
"Hotei",
"Hringhorni",
"Hubble",
"Hugi",
"Huron",
"Hyades",
"Hydrus",
"Hyksos",
"Hyperion",
"Hyperion",
"Iacchos",
"Iapetus",
"Ibanis",
"Icarus",
"Idun",
"Iki",
"Illiam",
"Illyria",
"Imperium",
"Indus",
"Infinity",
"Innana",
"Interigon",
"Inus",
"Io",
"Iolcus",
"Iota",
"Ioue",
"Irkalla",
"Ishtar",
"Ismene",
"Iswar",
"Iulus",
"Ixbalanque",
"Ixmirsis",
"Izahami",
"Jackal",
"Jade",
"Jamsid",
"Janus",
"Jataka",
"Jazdarnil",
"Jena",
"Jhurna",
"Jocasta",
"Jones",
"Jotars",
"Jotunheim",
"Jove",
"Juliet",
"Jung",
"Jupiter",
"Jurojin",
"K'unLun",
"Kachinas",
"Kadath",
"Kalki",
"Kamakura",
"Kant",
"Kappa",
"Karaja",
"Karnak",
"Kaus",
"Kay",
"Keats",
"Kenobi",
"Kepa",
"Keshvar",
"Kettoi",
"Khepri",
"Khwanirath",
"Kierkegaard",
"Kirin",
"Kirk",
"Kishijoten",
"Klingon",
"Kocab",
"Koffler",
"Kol",
"Kornophoros",
"Kryten",
"Kunmanggur",
"Kuya-Shonin",
"Kythamil",
"Labyrinth",
"Lachesis",
"Laindjung",
"Laius",
"Lamarna",
"Lambda",
"Lancelot",
"Lao-tze",
"Lapiths",
"Lares",
"Larissa",
"Larnok",
"Leda",
"Leia",
"Leibniz",
"Leinster",
"Lemnos",
"Leng",
"Leo",
"Leodegrance",
"Lepus",
"Lesath",
"Lethe",
"Leto",
"Leviathan",
"Lhard",
"Libra",
"Lister",
"Lodbrok",
"Lodur",
"Logan",
"Logi",
"Loki",
"Lot",
"Lotan",
"Love",
"Lucas",
"Lugundum",
"Lungshan",
"Luntag",
"Lupus",
"Luyia",
"Lycurgus",
"Lynx",
"Lyra",
"Lysithea",
"Macha",
"Mador",
"Maenads",
"Magellan",
"Magna",
"Mandara",
"Manitou",
"Marduk",
"Marmora",
"Mars",
"Mars",
"Mashyane",
"Mayhew",
"McCaffery",
"McCoy",
"Medea",
"Medusa",
"Meepzorp",
"Megara",
"Megnazon",
"Meneleus",
"Menrua",
"Merak",
"Mercury",
"Mercury",
"Merlin",
"Mesuta",
"Metis",
"Meukalinan",
"Miaplacidus",
"Midas",
"Midgard",
"Mimas",
"Mimir",
"Mimosa",
"Minoa",
"Minos",
"Mirach",
"Miranda",
"Mirfak",
"Miriam",
"Mirkwood",
"Mirzam",
"Mithrandir",
"Mizar",
"Mjolnir",
"Modred",
"Moirai",
"Monoceros",
"Montaigne",
"Mordor",
"Morrigan",
"Mulungu",
"Munin",
"Muphrid",
"Musca",
"Muspellheim",
"Nada",
"Nagilfar",
"Naiad",
"Nanna",
"Narcissus",
"Nardis",
"Nebo",
"Nelligan",
"Nemesis",
"Neptune",
"Neptune",
"Nereid",
"Nessus",
"Newton",
"Nietszche",
"Night",
"Ninhursag",
"Ninigi",
"Nirvana",
"Njord",
"Norn",
"Nu-ku",
"Nuada",
"Nuba",
"Nunka",
"Nuptadi",
"Nyambe",
"Nyarlathotep",
"Nymph",
"Nyx",
"Oberon",
"Obolus",
"Oceana",
"Od",
"Odin",
"Odysseus",
"Oedipus",
"Oeneus",
"Og",
"Ohrmazd",
"Ojin",
"Okesa",
"Okokanoffee",
"Okuninushi",
"Olifat",
"Oliphant",
"Olympus",
"Omega",
"Omicron",
"Oox",
"Ophelia",
"Ophiucus",
"Orchomenus",
"Ord",
"Orestes",
"Orion",
"Oro",
"Orpheus",
"Orthoprimax",
"Orunila",
"Osirus",
"Otus",
"Ovid",
"Oz",
"P'an-Ku",
"Pachacutil",
"Pan",
"Pandora",
"Parshya",
"Parvati",
"Pasiphae",
"Patala",
"Patrise",
"Pavo",
"Peace",
"Pegasus",
"Pek",
"Pelias",
"Pellas",
"Pellinore",
"Peloponnesus",
"Pelops",
"Percivale",
"Perseus",
"Persphone",
"Phakt",
"Phecda",
"Phegethon",
"Pheonix",
"Phi",
"Phineas",
"Phobos",
"Phoebe",
"Phoenix",
"Phorcys",
"Phrixus",
"Phyrro",
"Picard",
"Pictor",
"Pifall",
"Piithar",
"Pindar",
"Pinel",
"Pisces",
"Planck",
"Planck",
"Plato",
"Plato",
"Pleides",
"Pluto",
"Pluto",
"Polaris",
"Pollux",
"Polybus",
"Polydectes",
"Polyneices",
"Poseiden",
"Priam",
"Primax",
"Prithivi",
"Procyon",
"Proetus",
"Prometheus",
"Psi",
"Pthura",
"Ptolemy",
"Ptolemy",
"Puck",
"Puppis",
"Pymeria",
"Pyrgos-Dirou",
"Pyrrha",
"Pythia",
"Pyxis",
"QaQwaai",
"Qrdus",
"Quadran",
"QuagKeep",
"Quetzalcoatl",
"Quimbaya",
"Quipus",
"Quirnok",
"R'lyeh",
"Radha",
"Ragnarok",
"Rashnu",
"Rastaban",
"Rata",
"Reggae",
"Regia",
"Regulus",
"Regulus",
"Remus",
"Resida",
"Reticulum",
"Rhadamanthys",
"Rhea",
"Rho",
"Rhpisort",
"Rigel",
"Riker",
"Rimahad",
"Rimmer",
"Rol",
"Romulus",
"Rosalind",
"Rosenberg",
"Rotarev",
"Rousseau",
"Rudra",
"Ruhe",
"Rukmini",
"Runes",
"Rustam",
"Ryobu",
"Saberhagen",
"Sabik",
"Sadalrud",
"Sagittarius",
"Sagurak",
"Sakra",
"Salacia",
"Sandage",
"Sappedon",
"Sardonis",
"Sargas",
"Sarnak",
"Satan",
"Sati",
"Saturn",
"Saturn",
"Sauron",
"Scearce",
"Schopenhaur",
"Scorpio",
"Scott",
"Scylla",
"Secundus",
"Sedaria",
"Sedna",
"Sega",
"Sekasis",
"Semele",
"Seneca",
"Seriphos",
"Serpens",
"Sha-Lana",
"Shaman",
"Shantak",
"Shapash",
"Shapley",
"Shaula",
"Shax",
"Sheat",
"Shen",
"Siddhartha",
"Sif",
"Sig",
"Sigma",
"Sigyn",
"Simmons",
"Sinope",
"Sirens",
"Sirius",
"Skidbladnir",
"Skirnir",
"Skrymir",
"Skuld",
"Skywalker",
"Sleipner",
"Socrates",
"Sol",
"Solaria",
"Sphinx",
"Spica",
"Spielberg",
"Spinner",
"Spinoza",
"Spock",
"Srishok",
"Styx",
"Suboceana",
"Suga",
"Sugriva",
"Sulu",
"Surt",
"Surya",
"Swarga",
"Symplegades",
"Syrinx",
"T'aishan",
"Tacitus",
"Tadanobu",
"Tammuz",
"Tantrus",
"Tarazed",
"Tare",
"Tarkus",
"Tarnesus",
"Tarsus ",
"Tartarus",
"Taurus",
"Taweskare",
"Tekeli-li",
"Telesto",
"Teller",
"Terl",
"Terminus",
"Terra",
"Tethys",
"Thalassa",
"Thebes",
"Themis",
"Thera",
"Theseus",
"Theta",
"Thinis",
"Thjalfi",
"Thjazi",
"Thor",
"Thought",
"Thrace",
"Thrasymachus",
"Thuban",
"Ti'i",
"Tigra-nog'th",
"Tiki",
"Tillich",
"Timaeus",
"Time",
"Tirawa-Atius",
"Titan",
"Tiu",
"Tjinimin",
"Tolkien",
"Tou-Mu",
"Trantor",
"Trashtri",
"Triangulum",
"Trinus",
"Trisala",
"Triton",
"Troll",
"Troy",
"Tsathoggua",
"Tsentsa",
"Tucana",
"Typhon",
"Tyr",
"Tyrns",
"Tyrrea",
"Uhura",
"Ukaipu",
"Ulka",
"Umbriel",
"Unicorn",
"Unok",
"Unukalkay",
"Upsilon",
"Upuaut",
"Uranus",
"Uranus",
"Urd",
"Urdanis",
"Urech",
"Ursa",
"Ursa",
"Ursharabi",
"Urth",
"Urth",
"Utgard",
"Uther",
"Utnapishtim",
"Uxmal",
"Vader",
"Val",
"Valhalla",
"Valkyrie",
"VanCleef",
"Vanaheim",
"Vanir",
"Varnak",
"Vasuki",
"Ve",
"Vedas",
"Vedies",
"Vega",
"Vela",
"Venatici",
"Venus",
"Venus",
"Verdandi",
"Vesta",
"Vigari",
"Vili",
"Virag",
"Virgo",
"Vishtaspa",
"Vivian",
"VohuManuh",
"Volans",
"Voltaire",
"Volva",
"Vucu-Caquix",
"Vulcan",
"Vurat",
"Walla",
"Wampum",
"War",
"Waterloo",
"Waura",
"Wawalag",
"Wesen",
"Williams",
"Wirth",
"Woden",
"Woosley",
"Worf",
"Wu",
"Xantor",
"Xerxes",
"Xi",
"Xrka",
"Yaddith",
"Yami",
"Yamotodake",
"Yang",
"Yangtse",
"Yarnek",
"Ygerne",
"Yggdrasil",
"Yima",
"Yin",
"Ymir",
"Yoda",
"Yog-sothoth",
"Yrga",
"Yu-Nu",
"Yu-ti",
"Yucatan",
"Yuggoth",
"YumCaax",
"Zaaxon",
"Zahak",
"Zapana",
"Zarathrusta",
"Zarg",
"Zawijah",
"Zeta",
"Zetes",
"Zethus",
"Zeus",
"Zocho",
"Zog",
"Zoroaster",
"Zot",
"Zozca",
"Zrd",
"Zuben",
"Zurvan"
};
