/* Version of Celestial Empire */
/* (c) 1992 Dougal Scott */

/* $Header: /nelstaff/edp/dwagon/rfs/RCS/version.h,v 1.46 1993/07/08 03:28:46 dwagon Exp $ */
/* $Log: version.h,v $
 * Revision 1.46  1993/07/08  03:28:46  dwagon
 * Changed neutral to plr0. This changed most files.
 *
 * Revision 1.45  1993/05/20  03:22:59  dwagon
 * Update patchlevel by 1 to 1.45
 *
 * Revision 1.44  1992/09/16  14:03:46  dwagon
 * Initial RCS'd version
 * */

#define VERSION	1
#define PATCHLEVEL 46
