/* Celestial Empire (c) 1992 */
/* List of all the alliance names */
/* $Header: /nelstaff/edp/dwagon/rfs/RCS/allname.h,v 1.44 1992/09/16 13:49:11 dwagon Exp $ */
/* $Log: allname.h,v $
 * Revision 1.44  1992/09/16  13:49:11  dwagon
 * Initial RCS'd version
 *
 * Revision 1.1  1992/09/16  13:43:56  dwagon
 * Initial revision
 * */

static char allname[][10] = {"HOSTILE","NEUTRAL","FRIENDLY","ALLY" };
